Index: src/sys/sys/socketvar.h
===================================================================
RCS file: /home/chs/netbsd/cvs/src/sys/sys/socketvar.h,v
retrieving revision 1.162
diff -u -p -r1.162 socketvar.h
--- src/sys/sys/socketvar.h	17 Nov 2020 03:22:33 -0000	1.162
+++ src/sys/sys/socketvar.h	18 Nov 2020 19:54:42 -0000
@@ -197,12 +197,12 @@ struct socket {
 #define	SS_RCVATMARK		0x040	/* at mark on input */
 #define	SS_ISABORTING		0x080	/* aborting fd references - close() */
 #define	SS_RESTARTSYS		0x100	/* restart blocked system calls */
-#define	SS_ISDISCONNECTED	0x800	/* socket disconnected from peer */
-
+#define	SS_POLLRDBAND		0x200	/* poll should return POLLRDBAND */
 #define	SS_MORETOCOME		0x400	/*
 					 * hint from sosend to lower layer;
 					 * more data coming
 					 */
+#define	SS_ISDISCONNECTED	0x800	/* socket disconnected from peer */
 #define	SS_ISAPIPE 		0x1000	/* socket is implementing a pipe */
 #define	SS_NBIO			0x2000	/* socket is in non blocking I/O */
 
Index: src/sys/kern/uipc_socket.c
===================================================================
RCS file: /home/chs/netbsd/cvs/src/sys/kern/uipc_socket.c,v
retrieving revision 1.292
diff -u -p -r1.292 uipc_socket.c
--- src/sys/kern/uipc_socket.c	17 Oct 2020 09:06:15 -0000	1.292
+++ src/sys/kern/uipc_socket.c	18 Nov 2020 20:04:24 -0000
@@ -1186,9 +1186,6 @@ soreceive(struct socket *so, struct mbuf
 			    MIN(uio->uio_resid, m->m_len), uio);
 			m = m_free(m);
 		} while (uio->uio_resid > 0 && error == 0 && m);
-		/* We consumed the oob data, no more oobmark.  */
-		so->so_oobmark = 0;
-		so->so_state &= ~SS_RCVATMARK;
 bad:
 		if (m != NULL)
 			m_freem(m);
@@ -2214,6 +2211,7 @@ void
 sohasoutofband(struct socket *so)
 {
 
+	so->so_state |= SS_POLLRDBAND;
 	fownsignal(so->so_pgid, SIGURG, POLL_PRI, POLLPRI|POLLRDBAND, so);
 	selnotify(&so->so_rcv.sb_sel, POLLPRI | POLLRDBAND, NOTE_SUBMIT);
 }
@@ -2388,7 +2386,7 @@ sodopoll(struct socket *so, int events)
 			revents |= events & (POLLOUT | POLLWRNORM);
 
 	if (events & (POLLPRI | POLLRDBAND))
-		if (so->so_oobmark || (so->so_state & SS_RCVATMARK))
+		if (so->so_state & SS_POLLRDBAND)
 			revents |= events & (POLLPRI | POLLRDBAND);
 
 	return revents;
Index: src/sys/netinet/tcp_usrreq.c
===================================================================
RCS file: /home/chs/netbsd/cvs/src/sys/netinet/tcp_usrreq.c,v
retrieving revision 1.227
diff -u -p -r1.227 tcp_usrreq.c
--- src/sys/netinet/tcp_usrreq.c	17 Oct 2020 08:50:38 -0000	1.227
+++ src/sys/netinet/tcp_usrreq.c	18 Nov 2020 20:00:34 -0000
@@ -1095,8 +1095,10 @@ tcp_recvoob(struct socket *so, struct mb
 
 	m->m_len = 1;
 	*mtod(m, char *) = tp->t_iobc;
-	if ((flags & MSG_PEEK) == 0)
+	if ((flags & MSG_PEEK) == 0) {
 		tp->t_oobflags ^= (TCPOOB_HAVEDATA | TCPOOB_HADDATA);
+		so->so_state &= ~SS_POLLRDBAND;
+	}
 
 	tcp_debug_trace(so, tp, ostate, PRU_RCVOOB);
 	splx(s);
